import configparser
import tempfile
from contextlib import contextmanager
from pathlib import Path

import click

from send_to_kindle.converter import html_to_mobi
from send_to_kindle.downloader import download_images, get_article
from send_to_kindle.sender.email_sender import EmailSender


def get_config_path(config_file):
    if config_file is None:
        config_file = Path(Path.home(), "send_to_kindle.ini").resolve()
    else:
        config_file = Path(config_file)
    if not config_file.exists():
        raise ValueError("The configuration file does not exist")
    return config_file


def get_config(config_file):
    config_file = get_config_path(config_file)
    parser = configparser.ConfigParser()
    parser.read(str(config_file))
    return parser


@contextmanager
def write_temp_html(html, folder=None, delete=True):
    try:
        temp_html = tempfile.NamedTemporaryFile(
            suffix=".html", mode="w", dir=folder, delete=delete
        )
        temp_html.write(html)
        temp_html.flush()
        yield Path(temp_html.name)
    finally:
        temp_html.close()


# pylint: disable=too-many-locals,too-many-arguments
@click.command()
@click.argument("url")
@click.option("--config", type=click.Path(exists=True, dir_okay=False), default=None)
@click.option(
    "--output", "-o", type=click.Path(exists=True, file_okay=False), default=None
)
@click.option("--keep", "-k", type=click.BOOL, default=False)
def download(url, config, output, keep):
    configuration = get_config(config)
    from_email = configuration.get("mail_account", "from")
    password = configuration.get("mail_account", "password")
    to_email = configuration.get("mail_account", "to")
    host = configuration.get("mail_server", "host")
    port = configuration.getint("mail_server", "port")
    use_tls = configuration.getboolean("mail_server", "use_tls", default=True)
    kindlegen_path = Path(configuration.get("kindlegen", "path"))

    article = get_article(url)

    # pylint: disable=bad-continuation
    with write_temp_html(
        article.to_html().prettify(), folder=output, delete=not keep
    ) as html:
        download_images(html.parent, article.image_map)
        mobi_file = html_to_mobi(kindlegen_path, html, article.title)

    sender = EmailSender(from_email, password, host, port, use_tls)
    sender.send_mail(article.title, to_email, mobi_file)


if __name__ == "__main__":
    download()  # pylint: disable=no-value-for-parameter



import shutil
from pathlib import Path

import requests
from PIL import Image


def download_images(folder, image_map):
    for image_id, img_url in image_map.items():
        image_path = Path(folder, image_id)
        # pylint: disable=bad-continuation
        with open(image_path, "wb") as output_file, requests.get(
            img_url, stream=True
        ) as response:
            shutil.copyfileobj(response.raw, output_file)
        image = Image.open(str(image_path.resolve()))
        rgb_im = image.convert("RGB")
        rgb_im.save(str(image_path.resolve()))



from pathlib import Path

import requests
import tldextract
from bs4 import BeautifulSoup

from send_to_kindle.downloader.article import Article
from send_to_kindle.downloader.content_extractor import (
    ContentExtractor,
    DevToExtractor,
    MediumExtractor,
)

ROOT = Path(__file__).parent.parent.resolve()
TEMPLATE = Path(ROOT, "assets", "article-template.html")
DEFAULT_EXTRACTOR = ContentExtractor()
REGISTERED_EXTRACTORS = {"medium.com": MediumExtractor(), "dev.to": DevToExtractor()}


def get_extractor(host):
    if host in REGISTERED_EXTRACTORS:
        return REGISTERED_EXTRACTORS[host]
    return DEFAULT_EXTRACTOR


def load_template():
    with open(TEMPLATE) as template_reader:
        article_template = BeautifulSoup(template_reader.read())
        return article_template


def extract_content(content_extractor, soup):
    return content_extractor.extract(soup)


def extract_images(content_extractor, article_soup):
    return content_extractor.replace_images(article_soup)


def get_article(url):
    tld_extractor = tldextract.TLDExtract(suffix_list_urls=None)
    url_parsed = tld_extractor(url)
    response = requests.get(url)
    soup = BeautifulSoup(response.text, "lxml")
    template = load_template()
    article = Article(url=url, title=soup.title.text.strip(), template=template)
    extractor = get_extractor(f"{url_parsed.domain}.{url_parsed.suffix}")
    soup = extract_content(extractor, soup)
    content, img_map = extract_images(extractor, soup)
    article.content = content
    article.image_map = img_map
    return article



import hashlib

from bs4 import BeautifulSoup

IMAGE_EXTENSION = "jpg"


class ContentExtractor:
    def extract(self, soup):
        return soup

    def replace_images(self, soup):
        new_soup = BeautifulSoup(str(soup), "lxml").find("article")
        img_map = dict()
        imgs = new_soup.find_all("img")
        for img in imgs:
            src = img.get("src")
            if src:
                new_id = hashlib.md5(src.encode("utf8")).hexdigest()  # nosec
                new_id = f"{new_id}.{IMAGE_EXTENSION}"
                img_map[new_id] = src
                img["src"] = new_id
        return new_soup, img_map


class DevToExtractor(ContentExtractor):
    def extract(self, soup):
        article = soup.find("article")
        article_actions = article.find("div", {"class": "article-actions"})
        article_actions.extract()

        title = article.find("header", {"class": "title"})
        h3s = title.find("h3")
        if h3s:
            _ = [anchor.extract() for anchor in h3s.find_all("a", recursive=False)]

        html_variant_wrapper = article.find("div", {"class": "html-variant-wrapper"})
        if html_variant_wrapper:
            html_variant_wrapper.extract()

        return article


class MediumExtractor(ContentExtractor):

    exclude_for_tag = {"img": {"src", "alt"}}

    def extract(self, soup):
        article = soup.find("article")
        self._remove_attrs(article)
        title_div = self._find_title_div(article)
        sub_title = self._find_subtitle(title_div)
        if sub_title:
            author_info = self._find_author_div(sub_title)
        else:
            author_info = self._find_author_div(title_div)
        author_info.extract()
        _ = [hr.extract() for hr in soup.find_all("hr")]
        for figure in article.find_all("figure"):
            tag = self._find_interesting_figure(figure)
            if tag and tag.name == "img":
                figure.replace_with(tag)
            else:
                figure.extract()
        return article

    def _find_interesting_figure(self, figure):
        if figure.find("iframe"):
            return None
        noscript = figure.find("noscript")
        if noscript:
            return noscript.find("img")
        return None

    def _remove_attrs(self, article):
        for tag in article.findAll(True):
            exclude = MediumExtractor.exclude_for_tag.get(tag.name, set())
            available_attrs = set(tag.attrs.keys())
            available_attrs.difference_update(exclude)
            for attr in available_attrs:
                tag.attrs.pop(attr)
        return article

    def _find_title_div(self, article):
        for section in article.find_all("section"):
            div = section.select("div > h1")
            if div:
                return div[0].parent
        return None

    def _find_subtitle(self, title_div):
        current = title_div.next
        while current:
            if current.name == "div":
                if current.find("h2"):
                    return current
                if current.find("div"):
                    return None
            current = current.next

    def _find_author_div(self, title_div):
        current = title_div.next
        while current:
            if current.name == "div":
                if current.find("div"):
                    return current
            current = current.next



from pathlib import Path
from subprocess import run  # nosec


def html_to_mobi(kindlegen_path, input_path, file_name):
    new_env = {"PATH": str(kindlegen_path.resolve())}

    result = run(
        ["kindlegen", "-o", f"{file_name}.mobi", str(input_path.resolve())],
        env=new_env,  # nosec
    )
    if not result.returncode == 0 and not result.returncode == 1:
        raise ValueError()

    return Path(input_path.parent, f"{file_name}.mobi")



class Article:
    def __init__(self, url, title, template):
        self.url = url
        self.title = title
        self.template = template
        self._content = None
        self._img_map = None

    @property
    def content(self):
        return self._content

    @content.setter
    def content(self, value):
        self._content = value

    @property
    def image_map(self):
        return self._img_map

    @image_map.setter
    def image_map(self, value):
        self._img_map = value

    def to_html(self):
        self.template.find("title").string = self.title
        self.template.body.insert(0, self.content)
        return self.template



from setuptools import setup, find_packages
from pathlib import Path
from setuptools.command.install import install
import sys
import os

root_dir = Path(__file__).parent
requirements = Path(root_dir, "requirements.txt")
requirements_dev = Path(root_dir, "requirements-dev.txt")
packages = find_packages(where=root_dir, exclude=["tests*"])

VERSION = "0.4.0"


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""

    description = "verify that the git tag matches our version"

    def run(self):
        tag = os.getenv("CURRENT_TAG")

        if tag != f"v{VERSION}":
            info = "Git tag: {0} does not match the version of this library: {1}".format(
                tag, VERSION
            )
            sys.exit(info)


def list_requirements(file_name):
    return [line.strip() for line in open(file_name)]


with open("README.md", "r") as readable:
    long_description = readable.read()

setup(
    name="s2kindle",
    author="Antonio Feregrino",
    author_email="antonio.feregrino@gmail.com",
    version=VERSION,
    packages=packages,
    description="Send web articles to your kindle!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/fferegrino/send-to-kindle",
    install_requires=list_requirements(requirements),
    tests_require=list_requirements(requirements_dev),
    entry_points="""
        [console_scripts]
        s2k=send_to_kindle.cli:download
    """,
    include_package_data=True,
    cmdclass={"verify": VerifyVersionCommand},
)



import smtplib
from email.encoders import encode_base64
from email.mime.multipart import MIMEBase, MIMEMultipart


class EmailSender:

    # pylint: disable=too-many-locals,too-many-arguments
    def __init__(self, from_email, password, host, port, use_tls):
        self.from_email = from_email
        self.password = password
        self.port = port
        self.host = host
        self.use_tls = use_tls

    def prepare_attachment(self, attachment_path):
        attachment = MIMEBase("application", "octet-stream")
        attachment.set_payload(open(attachment_path, "rb").read())
        encode_base64(attachment)
        attachment.add_header(
            "Content-Disposition", f'attachment; filename="{attachment_path.name}"'
        )
        return attachment

    def prepare_message(self, subject, to_email):
        msg = MIMEMultipart()
        msg["Subject"] = subject
        msg["To"] = to_email
        msg["From"] = self.from_email
        return msg

    def send_mail(self, subject, to_email, attachment_path):
        msg = self.prepare_message(subject, to_email)
        part = self.prepare_attachment(attachment_path)
        msg.attach(part)
        with smtplib.SMTP(self.host, self.port) as server:
            if self.use_tls:
                server.starttls()
            server.login(self.from_email, self.password)
            server.sendmail(self.from_email, to_email, msg.as_string())
